import java.awt.*;
import java.applet.*;

/**
 * Contains a static method which displays a binary tree node.
 * @author Till Krullmann
 * @version 1.0
 */
class BinaryTreeNodePainter
{
	/** Horizontal spacing between (the centers of) two adjacent tree leaves on the bottommost level. */
	public static final int SPACING = 50;
	/** Vertical spacing between two adjacent levels of nodes. */ 
	public static final int ROW_HEIGHT = 50;
	/** Radius of the circle representing a tree node. */
	public static final int NODE_RADIUS = 12;
	
	/** Color used to draw lines between nodes. */
	public static final Color LINE_COLOR = Color.black;
	/** Color used to draw the outline of a node. */
	public static final Color NODE_FORECOLOR = Color.black;
	/** Color used to fill a node. */
	public static final Color NODE_BACKCOLOR = new Color(95, 185, 255);
	/** Color used to fill an active node (i.e. a node that is visually emphasized). */
	public static final Color NODE_BACKCOLOR_EM = new Color(255, 115, 75);
	/** Color used to indicate a new node */
	public static final Color NODE_BACKCOLOR_NEW = new Color(45, 240, 165);
	
	/**
	 * Displays a binary tree node.
	 * @param g The <b>Graphics</b> object to paint upon. The node will be painted in the origin (0, 0),
	 * so the caller may need to transform the graphics object first.
	 * @param node The <b>BinaryTreeNode</b> object that is to be displayed.
	 * @param backColor The background color for the node.
	 */
	static void paintNode(Graphics g, BinaryTreeNode node, Color backColor)
	{
		// Paint the circle representing the node
		g.setColor(backColor);
		g.fillOval(- NODE_RADIUS, - NODE_RADIUS, 2 * NODE_RADIUS, 2 * NODE_RADIUS);

		// Draw the outline of the circle
		g.setColor(NODE_FORECOLOR);
		g.drawOval( - NODE_RADIUS, - NODE_RADIUS, 2 * NODE_RADIUS - 1, 2 * NODE_RADIUS - 1 );

		// Draw the caption
		String caption = node.getValue().toString();

		FontMetrics fm = g.getFontMetrics();
		Rectangle captionBounds = fm.getStringBounds(caption, g).getBounds();
		g.drawString( caption, - captionBounds.width / 2, captionBounds.height / 4 );
	}
	
	/**
	 * Displays a binary tree node and, recursively, all of its descendants.
	 * @param g The <b>Graphics</b> object to paint upon. The node will be painted in the origin (0, 0),
	 * so the caller may need to transform the graphics object first.
	 * @param node
	 * The <b>BinaryTreeNode</b> object that is to be displayed.
	 */
	public static void paintTree(Graphics g, BinaryTreeNode node)
	{
		int depth = node.getDepth();
		
		if (node.getLeftChild() != null)
		{
			// determine position of left child node
			Point posLeft = new Point(- (1 << (depth - 2)) * SPACING / 2, ROW_HEIGHT);
			
			// draw a line to the left child node
			g.setColor(LINE_COLOR);
			g.drawLine(0, 0, posLeft.x, posLeft.y);
			
			// Create a transformed "child" graphics object
			Graphics cg = g.create(posLeft.x, posLeft.y, 0, 0);
			cg.setClip(null);
			
			// paint the left child node
			paintTree(cg, node.getLeftChild());
			
			// free the child graphics
			cg.dispose();
		}
		
		if (node.getRightChild() != null)
		{
			// determine position of right child node
			Point posRight = new Point((1 << (depth - 2)) * SPACING / 2, ROW_HEIGHT);
			
			// draw a line to the right child node
			g.setColor(LINE_COLOR);
			g.drawLine(0, 0, posRight.x, posRight.y);
		
			// Create a transformed "child" graphics object
			Graphics cg = g.create(posRight.x, posRight.y, 0, 0);
			cg.setClip(null);
		
			// paint the right child node
			paintTree(cg, node.getRightChild());
		
			// free the child graphics
			cg.dispose();
		}
		
		paintNode(g, node, NODE_BACKCOLOR);
	}
}

class StdNodeStatusListener implements NodeStatusListener
{
	BinaryTreeNode rootNode;
	Applet parentApplet;
	Graphics graphics;
	
	final int SLEEP_INTERVAL = 1500;
	
	StdNodeStatusListener(BinaryTreeNode n, Applet a, Graphics g)
	{
		rootNode = n;
		parentApplet = a;
		graphics = g;
	}
	
	Point getNodePoint(BinaryTreeNode node)
	{
		Point p = new Point();
		
		BinaryTreeNode tempNode = rootNode;
		
		int cmp;
		do
		{
			cmp = node.getValue().compareTo(tempNode.getValue());
			int depth = tempNode.getDepth();
			
			if (cmp < 0)
			{
				p.setLocation(
					p.x - (1 << (depth - 2)) * BinaryTreeNodePainter.SPACING / 2,
					p.y + BinaryTreeNodePainter.ROW_HEIGHT);
				tempNode = tempNode.getLeftChild();
			}
			else if (cmp > 0)
			{
				p.setLocation(
					p.x + (1 << (depth - 2)) * BinaryTreeNodePainter.SPACING / 2,
					p.y + BinaryTreeNodePainter.ROW_HEIGHT);
				tempNode = tempNode.getRightChild();
			}
			
		}
		while (cmp != 0);
		
		return p;
	}
	
	public void emphasize(BinaryTreeNode node)
	{
		Point p = getNodePoint(node);
		
		Graphics cg = graphics.create(p.x, p.y, 0, 0);
		cg.setClip(null);
		
		BinaryTreeNodePainter.paintNode(cg, node, BinaryTreeNodePainter.NODE_BACKCOLOR_EM);
	}
	
	public void unEmphasize(BinaryTreeNode node)
	{
		Point p = getNodePoint(node);
	
		Graphics cg = graphics.create(p.x, p.y, 0, 0);
		cg.setClip(null);
	
		BinaryTreeNodePainter.paintNode(cg, node, BinaryTreeNodePainter.NODE_BACKCOLOR);
	}
	
	public void showStatus(String msg)
	{
		parentApplet.showStatus(msg);
	}
	
	public void sleep()
	{
		try
		{
			Thread.sleep(SLEEP_INTERVAL);
		}
		catch (InterruptedException ex)
		{ }
	}
	
}

public class BinaryTreeCanvas extends Canvas
{
	BinaryTreeNode root;
	
	public BinaryTreeNode getRoot() { return root; }
	
	public BinaryTreeCanvas()
	{
		super();
		root = null;
	}
	
	public BinaryTreeCanvas(BinaryTreeNode r)
	{
		super();
		root = r;
	}
	
	public void insert(Comparable v)
	{
		Graphics sg = getGraphics().create(getWidth() / 2, BinaryTreeNodePainter.NODE_RADIUS * 2, 0, 0);
		sg.setClip(null);
		
		NodeStatusListener status = new StdNodeStatusListener(root, (Applet) this.getParent(), sg);
		
		Graphics g = getGraphics().create(
			BinaryTreeNodePainter.NODE_RADIUS * 2,
			BinaryTreeNodePainter.NODE_RADIUS * 2, 0, 0);
		g.setClip(null);
		
		BinaryTreeNodePainter.paintNode(
			g,
			new BinaryTreeNode(v),
			BinaryTreeNodePainter.NODE_BACKCOLOR_NEW);
		g.dispose();
		status.sleep();
		
		if (root == null)
			root = new BinaryTreeNode(v);
		else
			root.insert(v, status);
		
		sg.dispose();
		repaint();
		
		status.showStatus("Insertion complete.");
	}
	
	public void remove(Comparable v)
	{
		Graphics sg = getGraphics().create(getWidth() / 2, BinaryTreeNodePainter.NODE_RADIUS * 2, 0, 0);
		sg.setClip(null);
		
		NodeStatusListener status = new StdNodeStatusListener(root, (Applet) this.getParent(), sg);
		
		Graphics g = getGraphics().create(
			BinaryTreeNodePainter.NODE_RADIUS * 2,
			BinaryTreeNodePainter.NODE_RADIUS * 2, 0, 0);
		g.setClip(null);

		BinaryTreeNodePainter.paintNode(
			g,
			new BinaryTreeNode(v),
			BinaryTreeNodePainter.NODE_BACKCOLOR_NEW);
		g.dispose();
		status.sleep();
		
		if (root != null)
		{
			root = root.remove(v, status);
			status.showStatus("Removal complete.");
		}
		else
			status.showStatus("Tree contains no nodes.");
			
		sg.dispose();
		repaint();
	}
	
	public void paint(Graphics g)
	{
		g.setColor(getBackground());
		g.fillRect(0, 0, getWidth(), getHeight());
		
		Graphics tg = g.create(getWidth() / 2, BinaryTreeNodePainter.NODE_RADIUS * 2, 0, 0);
		tg.setClip(null);
		
		BinaryTreeNodePainter.paintTree(tg, root);
	}
}