/**
 * Represents a binary tree node (i.e. a tree node with up to 2 child nodes).
 * @author Till Krullmann
 * @version 1.0
 */
public class BinaryTreeNode
{
	private Comparable value;
	private BinaryTreeNode leftChild, rightChild;
	
	public BinaryTreeNode(Comparable v)
	{
		value = v;
		leftChild = null;
		rightChild = null;
	}
	
	public Comparable getValue() { return value; }
	public void setValue(Comparable v) { value = v; }

	public BinaryTreeNode getLeftChild() { return leftChild; }
	public BinaryTreeNode getRightChild() { return rightChild; }

	protected void setLeftChild(BinaryTreeNode node) { leftChild = node; }
	protected void setRightChild(BinaryTreeNode node) { rightChild = node; }

	/**
	 * Determines the depth of the tree node, i.e. the number of levels below it.
	 * @return The depth of the node.
	 */
	public int getDepth()
	{
		return Math.max(
			(leftChild == null) ? 1 : (leftChild.getDepth() + 1),
			(rightChild == null) ? 1 : (rightChild.getDepth() + 1) );
	}

	/**
	 * Determines whether the tree node is a leaf, i.e. it has no child nodes.
	 * @return <b>true</b> if the node is a leaf, <b>false</b> otherwise.
	 */
	public boolean isLeaf()
	{
		return (leftChild == null) && (rightChild == null);
	}
	
	/** 
	 * Searches for a tree node among this node and its descendants.
	 * @param v Value of the tree node to search for.
	 * @return The <b>BinaryTreeNode</b> object, or <b>null</b> if no node has been found.
	 */
	public BinaryTreeNode find(Comparable v)
	{
		int cmp = v.compareTo(value);
	
		if (cmp < 0)
			return (leftChild != null) ? leftChild.find(v) : null;
		else if (cmp == 0)
			return this;
		else	/* cmp > 0 */
			return (rightChild != null) ? rightChild.find(v) : null;
	}
	
	protected void insertInternal(BinaryTreeNode node, NodeStatusListener status)
	{
		status.emphasize(this);
		
		if ( node.getValue().compareTo(value) <= 0 )
		{
			status.showStatus("Comparing: " + node.getValue().toString() + " <= " + value.toString());
			status.sleep();
			
			status.unEmphasize(this);
			
			if (leftChild == null)
				setLeftChild(node);
			else
				leftChild.insertInternal(node, status);
		}
		else
		{
			status.showStatus("Comparing: " + node.getValue().toString() + " > " + value.toString());
			status.sleep();
			
			status.unEmphasize(this);
			
			if (rightChild == null)
				setRightChild(node);
			else
				rightChild.insertInternal(node, status);
		}
	}
	
	/**
	 * Inserts a new node into the tree.
	 * @param v The value of the new node.
	 */
	public void insert(Comparable v, NodeStatusListener status)
	{
		insertInternal(new BinaryTreeNode(v), status);
	}
	
	public BinaryTreeNode remove(Comparable v, NodeStatusListener status)
	{
		status.emphasize(this);
		
		int cmp = v.compareTo(value);
	
		if (cmp < 0)
		{
			status.showStatus("Comparing: " + v.toString() + " < " + value.toString());
			status.sleep();
			
			if (leftChild != null)
			{
				status.unEmphasize(this);
				setLeftChild(leftChild.remove(v, status));
				return this;
			}
			else
				return this;
		}
	
		else if (cmp == 0)
		{
			status.showStatus("Comparing: " + v.toString() + " = " + value.toString());
			status.sleep();
			
			if (rightChild != null)
			{
				status.showStatus("Finding node with successor value...");
				
				BinaryTreeNode swapNodeParent = this;
				BinaryTreeNode swapNode = rightChild;
				
				status.unEmphasize(this);
				status.emphasize(swapNode);
				status.sleep();
				
				while (swapNode.getLeftChild() != null)
				{
					swapNodeParent = swapNode;
					swapNode = swapNodeParent.getLeftChild();
					
					status.unEmphasize(swapNodeParent);
					status.emphasize(swapNode);
					status.sleep();
				}
				
				value = swapNode.getValue();
				if (swapNodeParent == this)
					this.setRightChild(null);
				else
					swapNodeParent.setLeftChild(null);
				
				return this;
			}
			else
			{
				status.unEmphasize(this);
				status.emphasize(leftChild);
				status.showStatus("Node is removed and replaced by its left child.");
				status.sleep();
				
				return leftChild;
			}
		}
	
		else	// cmp > 0
		{
			status.showStatus("Comparing: " + v.toString() + " < " + value.toString());
			status.sleep();
			
			if (rightChild != null)
			{
				status.unEmphasize(this);
				setRightChild(rightChild.remove(v, status));
				return this;
			}
			else
				return this;
		}
	}
}
